@@ -7,7 +7,6 @@ https://github.com/stellar/stellar-protocol/blob/master/ecosystem/sep-0001.md
 """
 
 import os
-import sys
 
 import toml
 from django.conf import settings as django_settings
@@ -43,22 +42,18 @@ class PolarisPlainTextRenderer(BaseRenderer):
 
 @api_view(["GET"])
 @renderer_classes([PolarisPlainTextRenderer])
-def generate_toml(request: Request) -> Response:
+def generate_toml(_request: Request) -> Response:
     """Generate a TOML-formatted string"""
-    # Define the module variable to reference in-memory constants
-    # Check if we've already read the TOML contents
-    this = sys.modules[__name__]
-    if hasattr(this, "STELLAR_TOML_CONTENTS"):
-        return Response(this.STELLAR_TOML_CONTENTS, content_type="text/plain")
-
-    # Check for a static TOML file if the cache is empty and the TOML
-    # integration function is not replaced.
     if registered_toml_func is get_stellar_toml:
-        static_toml = finders.find("polaris/stellar.toml")
+        # integration function is not used, check to see if a static file is defined
+        static_toml = None
+        if settings.LOCAL_MODE:
+            static_toml = finders.find("polaris/local-stellar.toml")
+        if not static_toml:
+            static_toml = finders.find("polaris/stellar.toml")
         if static_toml:
             with open(static_toml) as f:
-                this.STELLAR_TOML_CONTENTS = f.read()
-            return Response(this.STELLAR_TOML_CONTENTS, content_type="text/plain")
+                return Response(f.read(), content_type="text/plain")
 
     # The anchor uses the registered TOML function, replaced or not
     toml_dict = {
@@ -67,19 +62,19 @@ def generate_toml(request: Request) -> Response:
             for asset in Asset.objects.exclude(distribution_seed__isnull=True)
         ],
         "VERSION": "0.1.0",
-        "SIGNING_KEY": settings.SIGNING_KEY,
         "NETWORK_PASSPHRASE": settings.STELLAR_NETWORK_PASSPHRASE,
     }
-    if "sep-24" in django_settings.POLARIS_ACTIVE_SEPS:
+    if "sep-24" in settings.ACTIVE_SEPS:
         toml_dict["TRANSFER_SERVER"] = os.path.join(settings.HOST_URL, "sep24")
         toml_dict["TRANSFER_SERVER_SEP0024"] = toml_dict["TRANSFER_SERVER"]
-    if "sep-6" in django_settings.POLARIS_ACTIVE_SEPS:
+    if "sep-6" in settings.ACTIVE_SEPS:
         toml_dict["TRANSFER_SERVER"] = os.path.join(settings.HOST_URL, "sep6")
-    if "sep-10" in django_settings.POLARIS_ACTIVE_SEPS:
+    if "sep-10" in settings.ACTIVE_SEPS:
         toml_dict["WEB_AUTH_ENDPOINT"] = os.path.join(settings.HOST_URL, "auth")
-    if "sep-12" in django_settings.POLARIS_ACTIVE_SEPS:
+        toml_dict["SIGNING_KEY"] = settings.SIGNING_KEY
+    if "sep-12" in settings.ACTIVE_SEPS:
         toml_dict["KYC_SERVER"] = os.path.join(settings.HOST_URL, "kyc")
-    if "sep-31" in django_settings.POLARIS_ACTIVE_SEPS:
+    if "sep-31" in settings.ACTIVE_SEPS:
         toml_dict["DIRECT_PAYMENT_SERVER"] = os.path.join(settings.HOST_URL, "sep31")
 
     toml_dict.update(registered_toml_func())
