@@ -1,15 +1,22 @@
 from typing import Dict
-from polaris.utils import getLogger
 
 from django.utils.translation import gettext as _
-from django.core.exceptions import ObjectDoesNotExist
+from django.core.validators import URLValidator
+from django.core.exceptions import ObjectDoesNotExist, ValidationError
 from rest_framework.views import APIView
 from rest_framework.request import Request
 from rest_framework.response import Response
-from rest_framework.decorators import api_view, renderer_classes
-from rest_framework.renderers import JSONRenderer
+from rest_framework.decorators import api_view, renderer_classes, parser_classes
+from rest_framework.renderers import JSONRenderer, BrowsableAPIRenderer
+from rest_framework.parsers import JSONParser, MultiPartParser, FormParser
 
-from polaris.utils import extract_sep9_fields, render_error_response, memo_str
+from polaris import settings
+from polaris.utils import (
+    extract_sep9_fields,
+    render_error_response,
+    make_memo,
+    getLogger,
+)
 from polaris.sep10.utils import validate_sep10_token
 from polaris.integrations import registered_customer_integration as rci
 
@@ -18,26 +25,31 @@ logger = getLogger(__name__)
 
 
 class CustomerAPIView(APIView):
-    renderer_classes = [JSONRenderer]
+    renderer_classes = [JSONRenderer, BrowsableAPIRenderer]
+    parser_classes = [MultiPartParser, FormParser, JSONParser]
 
     @staticmethod
-    @validate_sep10_token("sep6")
+    @validate_sep10_token()
     def get(account: str, request: Request) -> Response:
         if request.GET.get("account") and account != request.GET.get("account"):
             return render_error_response(
                 _("The account specified does not match authorization token"),
                 status_code=403,
             )
-        elif not (request.GET.get("id") or request.GET.get("account")):
+        elif request.GET.get("id") and (
+            request.GET.get("account")
+            or request.GET.get("memo")
+            or request.GET.get("memo_type")
+        ):
             return render_error_response(
-                _("unable to identify a user without 'id' or 'account'")
+                _(
+                    "requests with 'id' cannot also have 'account', 'memo', or 'memo_type'"
+                )
             )
-        elif request.GET.get("memo_type") and not request.GET.get("memo"):
-            return render_error_response(_("missing 'memo' for 'memo_type'"))
 
         try:
             # validate memo and memo_type
-            memo_str(request.GET.get("memo"), request.GET.get("memo_type"))
+            make_memo(request.GET.get("memo"), request.GET.get("memo_type"))
         except ValueError:
             return render_error_response(_("invalid 'memo' for 'memo_type'"))
 
@@ -71,25 +83,32 @@ class CustomerAPIView(APIView):
         return Response(response_data)
 
     @staticmethod
-    @validate_sep10_token("sep6")
+    @validate_sep10_token()
     def put(account: str, request: Request) -> Response:
-        if request.data.get("id") and not request.data.get("account"):
+        if request.data.get("id"):
             if not isinstance(request.data.get("id"), str):
                 return render_error_response(_("bad ID value, expected str"))
+            elif (
+                request.data.get("account")
+                or request.data.get("memo")
+                or request.data.get("memo_type")
+            ):
+                return render_error_response(
+                    _(
+                        "requests with 'id' cannot also have 'account', 'memo', or 'memo_type'"
+                    )
+                )
         elif account != request.data.get("account"):
             return render_error_response(
                 _("The account specified does not match authorization token"),
                 status_code=403,
             )
-        elif request.data.get("memo_type") and not request.data.get("memo"):
-            return render_error_response(_("missing 'memo' for 'memo_type'"))
 
-        if request.data.get("memo"):
-            try:
-                # validate memo and memo_type
-                memo_str(request.data.get("memo"), request.data.get("memo_type"))
-            except ValueError:
-                return render_error_response(_("invalid 'memo' for 'memo_type'"))
+        try:
+            # validate memo and memo_type
+            make_memo(request.data.get("memo"), request.data.get("memo_type"))
+        except ValueError:
+            return render_error_response(_("invalid 'memo' for 'memo_type'"))
 
         try:
             customer_id = rci.put(
@@ -98,6 +117,7 @@ class CustomerAPIView(APIView):
                     "account": account,
                     "memo": request.data.get("memo"),
                     "memo_type": request.data.get("memo_type"),
+                    "type": request.data.get("type"),
                     **extract_sep9_fields(request.data),
                 }
             )
@@ -115,22 +135,82 @@ class CustomerAPIView(APIView):
         return Response({"id": customer_id}, status=202)
 
 
-@api_view(["DELETE"])
-@renderer_classes([JSONRenderer])
-@validate_sep10_token("sep6")
-def delete(account_from_auth: str, request: Request, account: str) -> Response:
-    if account_from_auth != account:
-        return render_error_response("account not found", status_code=404)
+@api_view(["PUT"])
+@renderer_classes([JSONRenderer, BrowsableAPIRenderer])
+@parser_classes([MultiPartParser, FormParser, JSONParser])
+@validate_sep10_token()
+def callback(account: str, request: Request) -> Response:
+    if request.data.get("id"):
+        if not isinstance(request.data.get("id"), str):
+            return render_error_response(_("bad ID value, expected str"))
+        elif (
+            request.data.get("account")
+            or request.data.get("memo")
+            or request.data.get("memo_type")
+        ):
+            return render_error_response(
+                _(
+                    "requests with 'id' cannot also have 'account', 'memo', or 'memo_type'"
+                )
+            )
+    elif account != request.data.get("account"):
+        return render_error_response(
+            _("The account specified does not match authorization token"),
+            status_code=403,
+        )
+
     try:
-        memo = memo_str(request.data.get("memo"), request.data.get("memo_type"))
+        # validate memo and memo_type
+        make_memo(request.data.get("memo"), request.data.get("memo_type"))
+    except ValueError:
+        return render_error_response(_("invalid 'memo' for 'memo_type'"))
+
+    callback_url = request.data.get("url")
+    if not callback_url:
+        return render_error_response(_("callback 'url' required"))
+    schemes = ["https"] if not settings.LOCAL_MODE else ["https", "http"]
+    try:
+        URLValidator(schemes=schemes)(request.data.get("url"))
+    except ValidationError:
+        return render_error_response(_("'url' must be a valid URL"))
+
+    try:
+        rci.callback(
+            {
+                "id": request.data.get("id"),
+                "account": account,
+                "memo": request.data.get("memo"),
+                "memo_type": request.data.get("memo_type"),
+                "url": callback_url,
+            }
+        )
     except ValueError as e:
-        return render_error_response("invalid 'memo' for 'memo_type'")
+        return render_error_response(str(e), status_code=400)
+    except ObjectDoesNotExist as e:
+        return render_error_response(str(e), status_code=404)
+    except NotImplementedError:
+        return render_error_response(_("not implemented"), status_code=501)
+
+    return Response({"success": True})
+
+
+@api_view(["DELETE"])
+@renderer_classes([JSONRenderer, BrowsableAPIRenderer])
+@parser_classes([MultiPartParser, FormParser, JSONParser])
+@validate_sep10_token()
+def delete(account_from_auth: str, request: Request, account: str,) -> Response:
+    if account_from_auth != account:
+        return render_error_response(_("account not found"), status_code=404)
     try:
-        rci.delete(account, memo, request.data.get("memo_type"))
+        make_memo(request.data.get("memo"), request.data.get("memo_type"))
+    except ValueError:
+        return render_error_response(_("invalid 'memo' for 'memo_type'"))
+    try:
+        rci.delete(account, request.data.get("memo"), request.data.get("memo_type"))
     except ObjectDoesNotExist:
-        return render_error_response("account not found", status_code=404)
+        return render_error_response(_("account not found"), status_code=404)
     else:
-        return Response({}, status=200)
+        return Response({"status": "success"}, status=200)
 
 
 def validate_response_data(data: Dict):
