@@ -5,20 +5,26 @@ non-Stellar-based account.
 """
 from decimal import Decimal, DecimalException
 from urllib.parse import urlencode
-from polaris.utils import getLogger
 
 from django.urls import reverse
+from django.core.exceptions import ValidationError
 from django.views.decorators.clickjacking import xframe_options_exempt
 from django.shortcuts import redirect
 from django.utils.translation import gettext as _
-from rest_framework.decorators import api_view, renderer_classes
+from rest_framework.decorators import api_view, renderer_classes, parser_classes
 from rest_framework.response import Response
 from rest_framework.request import Request
-from rest_framework.renderers import TemplateHTMLRenderer, JSONRenderer
+from rest_framework.renderers import (
+    TemplateHTMLRenderer,
+    JSONRenderer,
+    BrowsableAPIRenderer,
+)
+from rest_framework.parsers import MultiPartParser, FormParser, JSONParser
 
 from polaris import settings
 from polaris.templates import Template
 from polaris.utils import (
+    getLogger,
     render_error_response,
     extract_sep9_fields,
     create_transaction_id,
@@ -50,6 +56,7 @@ logger = getLogger(__name__)
 @xframe_options_exempt
 @api_view(["POST"])
 @renderer_classes([TemplateHTMLRenderer])
+@parser_classes([MultiPartParser, FormParser, JSONParser])
 @check_authentication()
 def post_interactive_withdraw(request: Request) -> Response:
     """
@@ -82,7 +89,7 @@ def post_interactive_withdraw(request: Request) -> Response:
     callback = args_or_error["callback"]
     amount = args_or_error["amount"]
 
-    form = rwi.form_for_transaction(transaction, post_data=request.POST)
+    form = rwi.form_for_transaction(transaction, post_data=request.data)
     if not form:
         logger.error(
             "Initial form_for_transaction() call returned None " f"for {transaction.id}"
@@ -102,7 +109,7 @@ def post_interactive_withdraw(request: Request) -> Response:
         )
 
     if not form.is_bound:
-        # The anchor must initialize the form with the request.POST data
+        # The anchor must initialize the form with request.data
         logger.error("form returned was not initialized with POST data, returning 500")
         return render_error_response(
             _("Unable to validate form submission."),
@@ -133,6 +140,7 @@ def post_interactive_withdraw(request: Request) -> Response:
                 f"Finished data collection and processing for transaction {transaction.id}"
             )
             invalidate_session(request)
+
             # Add memo now that interactive flow is complete
             #
             # We use the transaction ID as a memo on the Stellar transaction for the
@@ -144,13 +152,12 @@ def post_interactive_withdraw(request: Request) -> Response:
             transaction_id_hex = transaction.id.hex
             padded_hex_memo = "0" * (64 - len(transaction_id_hex)) + transaction_id_hex
             transaction.memo = memo_hex_to_base64(padded_hex_memo)
-            # Update status
-            # This signals to the wallet that the transaction can be submitted
             transaction.status = Transaction.STATUS.pending_user_transfer_start
             transaction.save()
-            url = reverse("more_info")
-            args = urlencode({"id": transaction.id, "callback": callback})
-            return redirect(f"{url}?{args}")
+            args = {"id": transaction.id, "initialLoad": "true"}
+            if callback:
+                args["callback"] = callback
+            return redirect(f"{reverse('more_info')}?{urlencode(args)}")
 
     else:
         content = (
@@ -207,15 +214,19 @@ def complete_interactive_withdraw(request: Request) -> Response:
         return render_error_response(
             _("Missing id parameter in URL"), content_type="text/html"
         )
-    Transaction.objects.filter(id=transaction_id).update(
-        status=Transaction.STATUS.pending_user_transfer_start
-    )
+    try:
+        Transaction.objects.filter(id=transaction_id).update(
+            status=Transaction.STATUS.pending_user_transfer_start
+        )
+    except ValidationError:
+        return render_error_response(
+            _("ID passed is not a valid transaction ID"), content_type="text/html"
+        )
     logger.info(f"Hands-off interactive flow complete for transaction {transaction_id}")
-    url, args = (
-        reverse("more_info"),
-        urlencode({"id": transaction_id, "callback": callback}),
-    )
-    return redirect(f"{url}?{args}")
+    args = {"id": transaction_id, "initialLoad": "true"}
+    if callback:
+        args["callback"] = callback
+    return redirect(f"{reverse('more_info')}?{urlencode(args)}")
 
 
 @xframe_options_exempt
@@ -248,6 +259,9 @@ def get_interactive_withdraw(request: Request) -> Response:
     asset = args_or_error["asset"]
     callback = args_or_error["callback"]
     amount = args_or_error["amount"]
+    if args_or_error["on_change_callback"]:
+        transaction.on_change_callback = args_or_error["on_change_callback"]
+        transaction.save()
 
     url = rwi.interactive_url(request, transaction, asset, amount, callback)
     if url:  # The anchor uses a standalone interactive flow
@@ -310,7 +324,8 @@ def get_interactive_withdraw(request: Request) -> Response:
 
 @api_view(["POST"])
 @validate_sep10_token()
-@renderer_classes([JSONRenderer])
+@renderer_classes([JSONRenderer, BrowsableAPIRenderer])
+@parser_classes([MultiPartParser, FormParser, JSONParser])
 def withdraw(account: str, request: Request) -> Response:
     """
     POST /transactions/withdraw/interactive
@@ -318,9 +333,9 @@ def withdraw(account: str, request: Request) -> Response:
     Creates an `incomplete` withdraw Transaction object in the database and
     returns the URL entry-point for the interactive flow.
     """
-    lang = request.POST.get("lang")
-    asset_code = request.POST.get("asset_code")
-    sep9_fields = extract_sep9_fields(request.POST)
+    lang = request.data.get("lang")
+    asset_code = request.data.get("asset_code")
+    sep9_fields = extract_sep9_fields(request.data)
     if lang:
         err_resp = validate_language(lang)
         if err_resp:
@@ -328,18 +343,13 @@ def withdraw(account: str, request: Request) -> Response:
         activate_lang_for_request(lang)
     if not asset_code:
         return render_error_response(_("'asset_code' is required"))
-    elif request.POST.get("memo"):
-        # Polaris SEP-24 doesn't support custodial wallets that depend on memos
-        # to disambiguate users using the same stellar account. Support would
-        # require new or adjusted integration points.
-        return render_error_response(_("`memo` parameter is not supported"))
 
     amount = None
-    if request.POST.get("amount"):
+    if request.data.get("amount"):
         try:
-            amount = Decimal(request.POST.get("amount"))
-        except DecimalException as e:
-            return render_error_response(_("Invalid 'amount'"))
+            amount = Decimal(request.data.get("amount"))
+        except DecimalException:
+            return render_error_response(_("invalid 'amount'"))
 
     # Verify that the asset code exists in our database, with withdraw enabled.
     asset = Asset.objects.filter(code=asset_code).first()
@@ -366,6 +376,9 @@ def withdraw(account: str, request: Request) -> Response:
         receiving_anchor_account=asset.distribution_account,
         memo_type=Transaction.MEMO_TYPES.hash,
         protocol=Transaction.PROTOCOL.sep24,
+        more_info_url=request.build_absolute_uri(
+            f"{reverse('more_info')}?id={transaction_id}"
+        ),
     )
     logger.info(f"Created withdrawal transaction {transaction_id}")
 
