@@ -1,8 +1,10 @@
 import pytest
 import json
-from unittest.mock import patch
+from unittest.mock import patch, Mock
 from typing import Dict
 
+from stellar_sdk import Keypair
+
 from polaris.models import Transaction
 from polaris.tests.helpers import mock_check_auth_success
 from polaris.integrations import DepositIntegration
@@ -19,20 +21,18 @@ class GoodDepositIntegration(DepositIntegration):
 
 
 @pytest.mark.django_db
-@patch("polaris.sep6.deposit.rdi", new_callable=GoodDepositIntegration)
-@patch("polaris.sep10.utils.check_auth", side_effect=mock_check_auth_success)
-def test_deposit_success(
-    mock_check, mock_integration, client, acc1_usd_deposit_transaction_factory
-):
-    del mock_check, mock_integration
-    deposit = acc1_usd_deposit_transaction_factory(protocol=Transaction.PROTOCOL.sep6)
-    asset = deposit.asset
+@patch("polaris.sep6.deposit.rdi", GoodDepositIntegration())
+@patch("polaris.sep10.utils.check_auth", mock_check_auth_success)
+def test_deposit_success(client, usd_asset_factory):
+    asset = usd_asset_factory(protocols=[Transaction.PROTOCOL.sep6])
     response = client.get(
-        DEPOSIT_PATH, {"asset_code": asset.code, "account": deposit.stellar_account},
+        DEPOSIT_PATH,
+        {"asset_code": asset.code, "account": Keypair.random().public_key},
     )
     content = json.loads(response.content)
     assert response.status_code == 200
     assert content == {
+        "id": str(Transaction.objects.first().id),
         "how": "test",
         "extra_info": {"test": "test"},
         "min_amount": round(asset.deposit_min_amount, asset.significant_decimals),
@@ -358,3 +358,107 @@ def test_saved_transaction_on_failure_response(client, usd_asset_factory):
         },
     )
     assert response.status_code == 500
+
+
+@pytest.mark.django_db
+@patch("polaris.sep6.deposit.rdi", GoodDepositIntegration())
+@patch("polaris.sep10.utils.check_auth", mock_check_auth_success)
+def test_claimable_balance_supported(client, usd_asset_factory):
+    asset = usd_asset_factory(protocols=[Transaction.PROTOCOL.sep6])
+    response = client.get(
+        DEPOSIT_PATH,
+        {
+            "asset_code": asset.code,
+            "account": Keypair.random().public_key,
+            "claimable_balance_supported": "true",
+        },
+    )
+    assert response.status_code == 200
+    t = Transaction.objects.first()
+    assert t.claimable_balance_supported
+
+
+@pytest.mark.django_db
+@patch("polaris.sep6.deposit.rdi", GoodDepositIntegration())
+@patch("polaris.sep10.utils.check_auth", mock_check_auth_success)
+def test_invalid_claimable_balance_supported_value(client, usd_asset_factory):
+    asset = usd_asset_factory(protocols=[Transaction.PROTOCOL.sep6])
+    response = client.get(
+        DEPOSIT_PATH,
+        {
+            "asset_code": asset.code,
+            "account": Keypair.random().public_key,
+            "claimable_balance_supported": "yes",
+        },
+    )
+    assert response.status_code == 400
+    assert "claimable_balance_supported" in json.loads(response.content)["error"]
+
+
+@pytest.mark.django_db
+@patch("polaris.sep6.deposit.rdi", GoodDepositIntegration())
+@patch("polaris.sep10.utils.check_auth", mock_check_auth_success)
+def test_claimable_balance_not_supported(client, usd_asset_factory):
+    asset = usd_asset_factory(protocols=[Transaction.PROTOCOL.sep6])
+    response = client.get(
+        DEPOSIT_PATH,
+        {
+            "asset_code": asset.code,
+            "account": Keypair.random().public_key,
+            "claimable_balance_supported": "False",
+        },
+    )
+    assert response.status_code == 200
+    t = Transaction.objects.first()
+    assert t.claimable_balance_supported is False
+
+
+@pytest.mark.django_db
+@patch("polaris.sep10.utils.check_auth", mock_check_auth_success)
+def test_bad_amount(client, usd_asset_factory):
+    asset = usd_asset_factory(protocols=[Transaction.PROTOCOL.sep6])
+    response = client.get(
+        DEPOSIT_PATH,
+        {
+            "asset_code": asset.code,
+            "account": Keypair.random().public_key,
+            "amount": "not an amount",
+        },
+    )
+    assert response.status_code == 400
+    assert "amount" in json.loads(response.content)["error"]
+
+
+@pytest.mark.django_db
+@patch("polaris.sep10.utils.check_auth", mock_check_auth_success)
+def test_amount_too_large(client, usd_asset_factory):
+    asset = usd_asset_factory(protocols=[Transaction.PROTOCOL.sep6])
+    response = client.get(
+        DEPOSIT_PATH,
+        {
+            "asset_code": asset.code,
+            "account": Keypair.random().public_key,
+            "amount": asset.deposit_max_amount + 1,
+        },
+    )
+    assert response.status_code == 400
+    assert "amount" in json.loads(response.content)["error"]
+
+
+@pytest.mark.django_db
+@patch("polaris.sep6.deposit.rdi")
+@patch("polaris.sep10.utils.check_auth", mock_check_auth_success)
+def test_good_amount(mock_deposit, client, usd_asset_factory):
+    asset = usd_asset_factory(protocols=[Transaction.PROTOCOL.sep6])
+    mock_deposit.process_sep6_request = Mock(return_value={"how": "test"})
+    response = client.get(
+        DEPOSIT_PATH,
+        {
+            "asset_code": asset.code,
+            "account": Keypair.random().public_key,
+            "amount": asset.deposit_max_amount - 1,
+        },
+    )
+    assert response.status_code == 200
+    args, _ = mock_deposit.process_sep6_request.call_args[0]
+    assert args.get("amount") == asset.deposit_max_amount - 1
