@@ -1,11 +1,10 @@
 import pytest
 import json
 from typing import Dict
-from unittest.mock import patch
+from unittest.mock import patch, Mock
 
 from stellar_sdk.keypair import Keypair
 
-from polaris import settings
 from polaris.tests.conftest import USD_DISTRIBUTION_SEED
 from polaris.tests.helpers import mock_check_auth_success
 from polaris.integrations import WithdrawalIntegration
@@ -26,14 +25,12 @@ class GoodWithdrawalIntegration(WithdrawalIntegration):
 @pytest.mark.django_db
 @patch("polaris.sep6.withdraw.rwi", GoodWithdrawalIntegration())
 @patch("polaris.sep10.utils.check_auth", mock_check_auth_success)
-def test_good_withdrawal_integration(client, acc1_usd_withdrawal_transaction_factory):
-    withdraw = acc1_usd_withdrawal_transaction_factory(
-        protocol=Transaction.PROTOCOL.sep6
-    )
+def test_good_withdrawal_integration(client, usd_asset_factory):
+    asset = usd_asset_factory(protocols=[Transaction.PROTOCOL.sep6])
     response = client.get(
         WITHDRAW_PATH,
         {
-            "asset_code": withdraw.asset.code,
+            "asset_code": asset.code,
             "type": "bank_account",
             "dest": "test bank account number",
         },
@@ -43,17 +40,12 @@ def test_good_withdrawal_integration(client, acc1_usd_withdrawal_transaction_fac
     assert content.pop("memo")
     assert content.pop("memo_type") == Transaction.MEMO_TYPES.hash
     assert content == {
+        "id": str(Transaction.objects.first().id),
         "account_id": Keypair.from_secret(USD_DISTRIBUTION_SEED).public_key,
-        "min_amount": round(
-            withdraw.asset.withdrawal_min_amount, withdraw.asset.significant_decimals
-        ),
-        "max_amount": round(
-            withdraw.asset.withdrawal_max_amount, withdraw.asset.significant_decimals
-        ),
-        "fee_fixed": round(
-            withdraw.asset.withdrawal_fee_fixed, withdraw.asset.significant_decimals
-        ),
-        "fee_percent": withdraw.asset.withdrawal_fee_percent,
+        "min_amount": round(asset.withdrawal_min_amount, asset.significant_decimals),
+        "max_amount": round(asset.withdrawal_max_amount, asset.significant_decimals),
+        "fee_fixed": round(asset.withdrawal_fee_fixed, asset.significant_decimals),
+        "fee_percent": asset.withdrawal_fee_percent,
         "extra_info": {"test": "test"},
     }
 
@@ -129,32 +121,22 @@ class MissingHowDepositIntegration(WithdrawalIntegration):
 @pytest.mark.django_db
 @patch("polaris.sep6.withdraw.rwi", MissingHowDepositIntegration())
 @patch("polaris.sep10.utils.check_auth", mock_check_auth_success)
-def test_withdraw_empty_integration_response(
-    client, acc1_usd_withdrawal_transaction_factory
-):
-    withdraw = acc1_usd_withdrawal_transaction_factory(
-        protocol=Transaction.PROTOCOL.sep6
-    )
+def test_withdraw_empty_integration_response(client, usd_asset_factory):
+    asset = usd_asset_factory(protocols=[Transaction.PROTOCOL.sep6])
     response = client.get(
-        WITHDRAW_PATH,
-        {"asset_code": withdraw.asset.code, "type": "good type", "dest": "test"},
+        WITHDRAW_PATH, {"asset_code": asset.code, "type": "good type", "dest": "test"},
     )
     content = json.loads(response.content)
     assert response.status_code == 200
     assert content.pop("memo")
     assert content.pop("memo_type") == Transaction.MEMO_TYPES.hash
     assert content == {
+        "id": str(Transaction.objects.first().id),
         "account_id": Keypair.from_secret(USD_DISTRIBUTION_SEED).public_key,
-        "min_amount": round(
-            withdraw.asset.withdrawal_min_amount, withdraw.asset.significant_decimals
-        ),
-        "max_amount": round(
-            withdraw.asset.withdrawal_max_amount, withdraw.asset.significant_decimals
-        ),
-        "fee_fixed": round(
-            withdraw.asset.withdrawal_fee_fixed, withdraw.asset.significant_decimals
-        ),
-        "fee_percent": withdraw.asset.withdrawal_fee_percent,
+        "min_amount": round(asset.withdrawal_min_amount, asset.significant_decimals),
+        "max_amount": round(asset.withdrawal_max_amount, asset.significant_decimals),
+        "fee_fixed": round(asset.withdrawal_fee_fixed, asset.significant_decimals),
+        "fee_percent": asset.withdrawal_fee_percent,
     }
 
 
@@ -240,16 +222,11 @@ def test_withdrawal_transaction_created(
         protocol=Transaction.PROTOCOL.sep6
     )
     distribution_address = Keypair.from_secret(USD_DISTRIBUTION_SEED).public_key
-    client.get(
+    response = client.get(
         WITHDRAW_PATH,
-        {
-            "asset_code": withdraw.asset.code,
-            "type": "good type",
-            "dest": "test",
-            "memo_type": "text",
-            "memo": "test",
-        },
+        {"asset_code": withdraw.asset.code, "type": "good type", "dest": "test",},
     )
+    assert response.status_code == 200
     t = (
         Transaction.objects.filter(kind=Transaction.KIND.withdrawal)
         .order_by("-started_at")
@@ -326,3 +303,60 @@ def test_saved_transaction_on_failure_response(client, usd_asset_factory):
         },
     )
     assert response.status_code == 500
+
+
+@pytest.mark.django_db
+@patch("polaris.sep10.utils.check_auth", mock_check_auth_success)
+def test_bad_amount(client, usd_asset_factory):
+    asset = usd_asset_factory(protocols=[Transaction.PROTOCOL.sep6])
+    response = client.get(
+        WITHDRAW_PATH,
+        {
+            "asset_code": asset.code,
+            "account": Keypair.random().public_key,
+            "type": "good type",
+            "amount": "not an amount",
+            "dest": "test bank account number",
+        },
+    )
+    assert response.status_code == 400
+    assert "amount" in json.loads(response.content)["error"]
+
+
+@pytest.mark.django_db
+@patch("polaris.sep10.utils.check_auth", mock_check_auth_success)
+def test_amount_too_large(client, usd_asset_factory):
+    asset = usd_asset_factory(protocols=[Transaction.PROTOCOL.sep6])
+    response = client.get(
+        WITHDRAW_PATH,
+        {
+            "asset_code": asset.code,
+            "account": Keypair.random().public_key,
+            "type": "good type",
+            "dest": "test bank account number",
+            "amount": asset.deposit_max_amount + 1,
+        },
+    )
+    assert response.status_code == 400
+    assert "amount" in json.loads(response.content)["error"]
+
+
+@pytest.mark.django_db
+@patch("polaris.sep6.withdraw.rwi")
+@patch("polaris.sep10.utils.check_auth", mock_check_auth_success)
+def test_good_amount(mock_deposit, client, usd_asset_factory):
+    asset = usd_asset_factory(protocols=[Transaction.PROTOCOL.sep6])
+    mock_deposit.process_sep6_request = Mock(return_value={"how": "test"})
+    response = client.get(
+        WITHDRAW_PATH,
+        {
+            "asset_code": asset.code,
+            "account": Keypair.random().public_key,
+            "type": "good type",
+            "dest": "test bank account number",
+            "amount": asset.deposit_max_amount - 1,
+        },
+    )
+    assert response.status_code == 200
+    args, _ = mock_deposit.process_sep6_request.call_args[0]
+    assert args.get("amount") == asset.deposit_max_amount - 1
